/* eslint-disable @typescript-eslint/no-unused-vars */
import { readFile, readdir, access } from "fs/promises";
import path from "path";

import matter from "gray-matter";
import { Client } from "ldapts";
import { serialize } from "next-mdx-remote/serialize";

import { getCurrentTerm } from "@/lib/events";

const EXECS_PATH = path.join("content", "team", "execs");
const fileType = ".md";
const { year, term } = getCurrentTerm();

export interface execMembers {
  name: string;
  order: string;
}

export interface Metadata {
  name: string;
  role: string;
  image: string;
}

export async function getExecNames() {
  if (process.env.USE_LDAP?.toLowerCase() !== "true") {
    return (await readdir(EXECS_PATH))
      .filter((name) => name.endsWith(fileType))
      .map((name) => name.slice(0, -1 * fileType.length));
  }

  const url = "ldap://ldap1.csclub.uwaterloo.ca";
  const searchDN = "ou=People,dc=csclub,dc=uwaterloo,dc=ca";
  const client = new Client({ url });

  try {
    await client.bind("", "");
    const { searchEntries } = await client.search(searchDN, {
      scope: "sub",
      filter: `(&(objectClass=member)(term=${(term as string).slice(
        0,
        1
      )}${year})(exec=True))`,
    });

    let execMembers: execMembers[] = [];

    execMembers = searchEntries
      .map((item) => {
        return {
          name: item.cn as string,
          order: item.order as string,
        };
      })
      .sort((item1: execMembers, item2: execMembers) =>
        item1.order.localeCompare(item2.order)
      );

    const formattedExec: string[] = execMembers.map(
      (member) =>
        `0${member.order}-
        ${member.name.split(" ")[1].toLowerCase()}-
        ${member.name.split(" ")[0].toLowerCase()}`
    );
  } finally {
    await client.unbind();
  }

  return formattedExec;
}

export async function getExec(fileName: string, convert = true) {
  const raw = await readFile(path.join(EXECS_PATH, `${fileName}${fileType}`));
  const { content, data: metadata } = matter(raw);
  const image =
    (metadata.image as string | undefined) ??
    (await getMemberImagePath(metadata.name));

  return {
    content: convert ? await serialize(content) : content,
    metadata: { ...metadata, image } as Metadata,
  };
}

async function getImage(imgPath: string) {
  try {
    await access(path.join("public", imgPath));
    return imgPath;
  } catch {
    return undefined;
  }
}

export async function getMemberImagePath(name: string) {
  const imgPath = path.join("images", "team", name.replace(" ", ""));
  const placeholder = path.join(
    "images",
    "team",
    "team-member-placeholder.svg"
  );
  const img =
    (await getImage(imgPath + ".jpg")) ??
    (await getImage(imgPath + ".png")) ??
    (await getImage(imgPath + ".gif")) ??
    (await getImage(imgPath + ".jpeg")) ??
    placeholder;
  return img;
}
